@@ -8,13 +8,14 @@ from collections import OrderedDict, defaultdict
 from conan.api.output import ConanOutput
 from conan.errors import ConanException
 from conan.internal.model.pkg_type import PackageType
-from conans.util.files import load, save
+from conan.internal.util.files import load, save
 
 _DIRS_VAR_NAMES = ["_includedirs", "_srcdirs", "_libdirs", "_resdirs", "_bindirs", "_builddirs",
                    "_frameworkdirs", "_objects"]
 _FIELD_VAR_NAMES = ["_system_libs", "_package_framework", "_frameworks", "_libs", "_defines",
-                    "_cflags", "_cxxflags", "_sharedlinkflags", "_exelinkflags"]
+                    "_cflags", "_cxxflags", "_sharedlinkflags", "_exelinkflags", "_sources"]
 _ALL_NAMES = _DIRS_VAR_NAMES + _FIELD_VAR_NAMES
+_SINGLE_VALUE_VARS = "_type", "_exe", "_location", "_link_location", "_languages"
 
 
 class MockInfoProperty:
@@ -81,6 +82,7 @@ class _Component:
         self._sharedlinkflags = None  # linker flags
         self._exelinkflags = None  # linker flags
         self._objects = None  # linker flags
+        self._sources = None  # source files
         self._exe = None  # application executable, only 1 allowed, following CPS
         self._languages = None
 
@@ -120,6 +122,7 @@ class _Component:
             "sharedlinkflags": self._sharedlinkflags,
             "exelinkflags": self._exelinkflags,
             "objects": self._objects,
+            "sources": self._sources,
             "sysroot": self._sysroot,
             "requires": self._requires,
             "properties": self._properties,
@@ -380,6 +383,16 @@ class _Component:
     def objects(self, value):
         self._objects = value
 
+    @property
+    def sources(self):
+        if self._sources is None:
+            self._sources = []
+        return self._sources
+
+    @sources.setter
+    def sources(self, value):
+        self._sources = value
+
     @property
     def sysroot(self):
         if self._sysroot is None:
@@ -449,6 +462,12 @@ class _Component:
                 else:
                     setattr(self, varname, other_values)
 
+        for varname in _SINGLE_VALUE_VARS:  # To allow editable of .exe/.location
+            other_values = getattr(other, varname)
+            if other_values is not None:
+                # Just overwrite the existing value, not possible to append
+                setattr(self, varname, other_values)
+
         if other.requires:
             current_values = self.get_init("requires", [])
             merge_list(other.requires, current_values)
@@ -817,7 +836,7 @@ class CppInfo:
                 if r not in ret:
                     ret.append(r)
         # Then split the names
-        ret = [r.split("::") if "::" in r else (None, r) for r in ret]
+        ret = [r.split("::", 1) if "::" in r else (None, r) for r in ret]
         return ret
 
     def deduce_full_cpp_info(self, conanfile):
@@ -826,8 +845,9 @@ class CppInfo:
 
         result = CppInfo()  # clone it
         if self.libs and len(self.libs) > 1:  # expand in multiple components
-            ConanOutput().warning(f"{conanfile}: The 'cpp_info.libs' contain more than 1 library. "
-                                  "Define 'cpp_info.components' instead.")
+            ConanOutput(scope=str(conanfile)).warning(
+                "The 'cpp_info.libs' contain more than 1 library. "
+                "Define 'cpp_info.components' instead.", warn_tag="deprecated")
             assert not self.components, f"{conanfile} cpp_info shouldn't have .libs and .components"
             common = self._package.clone()
             common.libs = []
@@ -846,7 +866,30 @@ class CppInfo:
         else:
             result._package = self._package.clone()
             result.default_components = self.default_components
-            result.components = {k: v.clone() for k, v in self.components.items()}
+            new_components = {}
+            for k, v in self.components.items():
+                if v.libs and len(v.libs) > 1:
+                    ConanOutput(scope=str(conanfile)).warning(
+                        f"The 'cpp_info.components[{k}] contains more than 1 library. "
+                        "Define 1 component for each library instead.", warn_tag="deprecated")
+                    # Now the root, empty one
+                    common = v.clone()
+                    common.libs = []
+                    common.type = str(PackageType.HEADER)  # the type of components is a string!
+                    new_components[k] = common
+
+                    for lib in v.libs:
+                        c = _Component()
+                        c.type = v.type
+                        c.includedirs = v.includedirs
+                        c.libdirs = v.libdirs
+                        c.bindirs = v.bindirs
+                        c.libs = [lib]
+                        new_components[f"_{k}_{lib}"] = c
+                        common.requires.append(f"_{k}_{lib}")
+                else:
+                    new_components[k] = v.clone()
+            result.components = new_components
 
         result._package.deduce_locations(conanfile)
         for comp_name, comp in result.components.items():
